@@ -47,11 +47,16 @@ setRefClass("RzMain",
       rzVariableEditorView$setInfo.bar(info.bar)
       plot.view$pack2(rzPlot$getMain(), resize=FALSE)
       if(rzSettings$getUseEmbededDevice()){
-        plot.area <- gtkDrawingArea()
-        plot.view$pack1(plot.area, resize=TRUE)
-        plot.view$setPosition(300)
-        asCairoDevice(plot.area)
-        rzSettings$setEmbededDeviceOn(TRUE)
+        if(require(cairoDevice)) {
+          plot.area <- gtkDrawingArea()
+          plot.view$pack1(plot.area, resize=TRUE)
+          plot.view$setPosition(300)
+          asCairoDevice(plot.area)
+          rzSettings$setEmbededDeviceOn(TRUE)
+        } else {
+          cat("cairoDevice package isn't installed", fill=TRUE)
+          rzSettings$setEmbededDeviceOn(FALSE)          
+        }
       } else {
         rzSettings$setEmbededDeviceOn(FALSE)
       }
@@ -73,6 +78,7 @@ setRefClass("RzMain",
       rzVariableEditorView$setAccel(accel.group)
       
       rzActionGroup$getA.plot.view()$setActive(rzSettings$getPlotViewEnabled())
+      rzActionGroup$getA.variable.editor.view()$setActive(rzSettings$getVariableEditorViewEnabled())
       rzActionGroup$getA.reload()$setSensitive(rzSettings$getUseDataSetObject())
       gSignalConnect(rzActionGroup$getA.open(),      "activate", .self$onOpen)
       gSignalConnect(rzActionGroup$getA.save(),      "activate", .self$onSave)
@@ -81,11 +87,17 @@ setRefClass("RzMain",
       gSignalConnect(rzActionGroup$getA.remove(),    "activate", .self$onRemove)
       gSignalConnect(rzActionGroup$getA.revert(),    "activate", .self$onRevert)
       gSignalConnect(rzActionGroup$getA.reload(),    "activate", .self$onReload)
+      gSignalConnect(rzActionGroup$getA.selectall(), "activate", .self$onSelectAll)
+      gSignalConnect(rzActionGroup$getA.unselect(),  "activate", .self$onUnselect)
+      gSignalConnect(rzActionGroup$getA.delete(),    "activate", .self$onDelete)
+      gSignalConnect(rzActionGroup$getA.duplicate(), "activate", .self$onDuplicate)
       gSignalConnect(rzActionGroup$getA.quit(),      "activate", win$destroy)
       gSignalConnect(rzActionGroup$getA.settings(),  "activate", .self$onSetting)
       gSignalConnect(rzActionGroup$getA.data.view(), "activate", .self$onDataView)
       gSignalConnect(rzActionGroup$getA.plot.view(), "toggled" , .self$onPlotViewToggled)
       gSignalConnect(rzActionGroup$getA.variable.editor.view(), "toggled" , .self$onVariableEditorViewToggled)
+      gSignalConnect(rzActionGroup$getA.tutorial(),  "activate", function(...) browseURL(gettext("http://m884.jp/RzTutorial.html")))
+      gSignalConnect(rzActionGroup$getA.load.sample(), "activate", .self$onLoadSample)
       gSignalConnect(rzActionGroup$getA.value.lab(), "activate", .self$onEditValueLabels)
       
       rzDataHandler <<- new("RzDataHandler", data.collection=data.collection.obj)
@@ -121,7 +133,36 @@ setRefClass("RzMain",
     
     # actions
     onEditValueLabels = function(action){
-      variable.view$onEditValueLabels(win=win)
+      if(is.null(variable.view)) return()
+      variable.view$onEditValueLabels()
+    },
+
+    onSelectAll = function(action){
+      if(is.null(variable.view)) return()
+      variable.view$onSelectAll()
+    },
+    
+    onUnselect = function(action){
+      if(is.null(variable.view)) return()
+      variable.view$onUnselect()
+    },
+    
+    onDelete = function(action){
+      if(is.null(variable.view)) return()
+      dialog <- gtkMessageDialogNew(win, "destroy-with-parent",
+                                    GtkMessageType["question"], GtkButtonsType["ok-cancel"],
+                                    gettext("Are you sure you want to do that?"))
+      response <- dialog$run()
+      dialog$hide()
+      
+      if(response==GtkResponseType["ok"]){
+        variable.view$onDelete()
+      }
+    },
+
+    onDuplicate = function(action){
+      if(is.null(variable.view)) return()
+      variable.view$onDuplicate()
     },
     
     onDataView = function(action){
@@ -198,12 +239,25 @@ setRefClass("RzMain",
     
     onImportFromGlobalEnv = function(action){
       timeoutid <- gTimeoutAdd(80, progress.bar$start)
-      gSourceRemove(timeoutid)
       data <- rzDataSetIO$importFromGlobalEnv(win)
-      progress.bar["activity-mode"] <<- FALSE
       if(!is.null(data)) {
         rzDataHandler$addData(data)
       }
+      gSourceRemove(timeoutid)
+      progress.bar["activity-mode"] <<- FALSE
+    },
+    
+    onLoadSample = function(action){
+      timeoutid <- gTimeoutAdd(80, progress.bar$start)
+#      data <- rzDataSetIO$importFromGlobalEnv(win)
+      nes1948.por <- UnZip("anes/NES1948.ZIP","NES1948.POR",package="memisc")
+      nes1948 <- spss.portable.file(nes1948.por)
+      sample.data.set <- as.data.set(nes1948)
+      data <- new("RzData", file.path=NULL, data.set=sample.data.set,
+                  original.name=gettext("NES1948 [Sample Dataset in memisc]"))
+      rzDataHandler$addData(data)
+      gSourceRemove(timeoutid)
+      progress.bar["activity-mode"] <<- FALSE
     },
     
     onChangeDataSetName   = function(action){
@@ -342,7 +396,7 @@ setRefClass("RzMain",
       if ( is.null(variable.view) ) {
         timeoutid <- gTimeoutAdd(80, progress.bar$start)
         variable.view <<- new("RzVariableView", data=rzDataHandler$getData(data.set.name),
-                              rzPlot=rzPlot)
+                              win=win, rzPlot=rzPlot)
         variable.view$construct()
         variable.view.list[[data.set.name]] <<- variable.view
         main.view$packStart(variable.view$getSw())
