@@ -1,14 +1,45 @@
 rzplot <- 
 setRefClass("RzPlot",
-  fields = c("win", "info.bar", "main",
-             "geom", "stat", "label", "stratum",
-             "facet", "position", "save",
-             "df", "p.current", "legend.position", "theme_Rz"),
+  fields = c("win", "info.bar", "main", "data", "model",
+             "geom", "stat", "label", "stratum", "button.prev", "button.next",
+             "facet", "position", "misc", "save",
+             "df", "p.current", "p.list", "p.current.num",
+             "legend.position", "legend.linetype", "legend.justification", "theme_Rz"),
   methods = list(
     initialize  = function(...) {
       initFields(...)
       
       p.current <<- NULL
+      model <<- NULL
+      data <<- NULL
+      p.list <<- list()
+      p.current.num <<- 1
+      
+      button.execute       <- gtkButtonNewFromStock(GTK_STOCK_EXECUTE)
+      
+      image  <- gtkImageNewFromStock(GTK_STOCK_CLEAR, GtkIconSize["button"])
+      button.clear          <- gtkButtonNew()
+      button.clear["sensitive"] <- TRUE
+      button.clear$setImage(image)
+      
+      image  <- gtkImageNewFromStock(GTK_STOCK_GO_BACK, GtkIconSize["button"])
+      button.prev          <<- gtkButtonNew()
+      button.prev["sensitive"] <<- FALSE
+      button.prev$setImage(image)
+      
+      image  <- gtkImageNewFromStock(GTK_STOCK_GO_FORWARD, GtkIconSize["button"])
+      button.next          <<- gtkButtonNew()
+      button.next["sensitive"] <<- FALSE
+      button.next$setImage(image)
+      
+      button.box.history <- gtkHBoxNew()
+      button.box.history$packStart(button.prev, expand=FALSE)
+      button.box.history$packStart(button.next, expand=FALSE)
+            
+      button.box1          <- gtkHBoxNew()
+      button.box1$packEnd(button.execute, padding=5, expand=FALSE)
+      button.box1$packEnd(button.box.history, expand=FALSE)
+      button.box1$packEnd(button.clear, padding=5, expand=FALSE)
       
       geom     <<- new("RzPlotGeom")
       stat     <<- new("RzPlotStat")
@@ -16,6 +47,7 @@ setRefClass("RzPlot",
       stratum  <<- new("RzPlotStratum")
       facet    <<- new("RzPlotFacet")
       label    <<- new("RzPlotLabel")
+      misc     <<- new("RzPlotMisc")
       save     <<- new("RzPlotSave", win=win)
       theme_Rz <<- theme_grey
       
@@ -28,27 +60,87 @@ setRefClass("RzPlot",
       vbox$packStart(position$getExpander(), expand=FALSE)
       vbox$packStart(facet$getExpander(), expand=FALSE)
       vbox$packStart(label$getExpander(), expand=FALSE)
+      vbox$packStart(misc$getExpander(), expand=FALSE)
       vbox$packStart(save$getExpander(), expand=FALSE)
       
-      main <<- gtkScrolledWindowNew()
-      main$setPolicy(GtkPolicyType["automatic"], GtkPolicyType["automatic"])
-      main$setShadowType(GtkShadowType["none"])
-      main$addWithViewport(vbox)
+      sw <- gtkScrolledWindowNew()
+      sw$setPolicy(GtkPolicyType["automatic"], GtkPolicyType["automatic"])
+      sw$setShadowType(GtkShadowType["none"])
+      sw$addWithViewport(vbox)
       
+      main <<- gtkVBoxNew()
+      main$packStart(button.box1, expand=FALSE, padding=5)
+      main$packStart(sw, expand=TRUE, fill=TRUE)
+      
+      
+      gSignalConnect(button.execute, "clicked", function(button){
+        if(!is.null(data)) .self$onPlot()
+      })
+      
+      gSignalConnect(button.clear, "clicked", function(button){
+        geom$clear()
+        stat$clear()
+        position$clear()
+        stratum$clear()
+        facet$clear()
+        label$clear()
+        misc$clear()
+      })
+      
+      gSignalConnect(button.prev, "clicked", function(button){
+        p.current.num <<- p.current.num + 1
+        p.current <<- p.list[[p.current.num]]
+        suppressWarnings(print(p.current))
+        button.next["sensitive"] <<- TRUE
+        if(p.current.num==length(p.list)) button["sensitive"] <- FALSE        
+      })
+
+      gSignalConnect(button.next, "clicked", function(button){
+        p.current.num <<- p.current.num - 1
+        p.current <<- p.list[[p.current.num]]
+        suppressWarnings(print(p.current))
+        button.prev["sensitive"] <<- TRUE
+        if(p.current.num==1) button["sensitive"] <- FALSE        
+      })
+
       gSignalConnect(save$getDialog(), "response", .self$onSave)
       gSignalConnect(geom$getButton(), "clicked",  .self$addGeom)
       gSignalConnect(stat$getButton(), "clicked",  .self$addStat)
     },
     
     onSave = function(dialog, response.id){
-      save$onSave(response.id, p.current, theme_Rz, legend.position)
+      save$onSave(response.id, p.current, theme_Rz, legend.position, legend.linetype, legend.justification)
     },
     
-    onPlot = function(data, col){
+    setX = function(col){
+      variableNames <- data$getVariableNames()
+      geom$setX(variableNames[col])
+    },
+    
+    onPlot = function(){
+      geom.args       <-  geom$getArgs()
+      stat.args       <-  stat$getArgs()
+      position.args   <-  position$getArgs()
+      stratum.args    <-  stratum$getArgs()
+      legend.position <<- stratum.args$legend.position
+      legend.linetype <<- stratum.args$legend.linetype
+      legend.justification <<- ifelse(length(legend.position)==2, c(0.5,0.5), c(0.5, 0.5))
+      facet.args      <-  facet$getArgs()
+      misc.args       <-  misc$getArgs()
+
       font <- rzSettings$getPlotFontFamily()
       data.frame <- data$getData.frame()
       variableLabels <- data$getVariableLabels()
       variableNames <- data$getVariableNames()
+      
+      col <- which(variableNames==geom.args$x)
+      if(length(col)==0){
+        info.bar$setMessageType(GtkMessageType["error"])
+        info.bar$setText(gettext("Please enter valid x."))
+        info.bar$show()
+        return()
+      }
+      
       df <<- data.frame[col]
       colnames(df) <<- "x"
       
@@ -56,40 +148,38 @@ setRefClass("RzPlot",
       title <- label.args$title
       xlab  <- label.args$xlab
       ylab  <- label.args$ylab
-      
-      geom.args       <-  geom$getArgs()
-      stat.args       <-  stat$getArgs()
-      position.args   <-  position$getArgs()
-      stratum.args    <-  stratum$getArgs()
-      legend.position <<- stratum.args$legend.position
-      facet.args      <- facet$getArgs()
-      
+            
       y <- NULL
       if(!is.null(geom.args$y)){
         y <- geom.args$y
         df$y <<- data.frame[[y]]        
       }
       
-      if (geom.args$theme=="gray") {
+      if (misc.args$theme=="grey") {
         theme_Rz <<- theme_grey
-      } else if (geom.args$theme=="bw") {
+      } else if (misc.args$theme=="bw") {
         theme_Rz <<- theme_bw        
       }
       
       position.layer <- switch(position.args$position,
+                               default  = NULL,
                                identity = position_identity(width=position.args$width, height= position.args$height),
                                dodge    = position_dodge   (width=position.args$width, height= position.args$height),
                                fill     = position_fill    (width=position.args$width, height= position.args$height),
                                stack    = position_stack   (width=position.args$width, height= position.args$height),
                                jitter   = position_jitter  (width=position.args$width, height= position.args$height))
       
-      
+      group <- NULL
+      fill  <- NULL
+      color <- NULL
+      shape <- NULL
+      size  <- NULL
+      line  <- NULL
       if(!is.null(stratum.args$group )){
-        df$group  <<- data.frame[[stratum.args$group]]
-        if(stratum.args$group.label==gettext("variable label")){
-          stratum.args$group.label <- variableLabels[which(stratum.args$group==variableNames)]
-        } else if(stratum.args$group.label==gettext("variable name")){
-          stratum.args$group.label <- stratum.args$group
+        group <- stratum.args$group
+        df$group <<- data.frame[[group]]
+        if(is.null(df$group)){
+          group <- 1
         }
       }
       if(!is.null(stratum.args$fill )){
@@ -122,8 +212,15 @@ setRefClass("RzPlot",
           stratum.args$size.label <- variableLabels[which(stratum.args$size==variableNames)]
         } else if(stratum.args$size.label==gettext("variable name")){
           stratum.args$size.label <- stratum.args$size
-        }
-        
+        }        
+      }
+      if(!is.null(stratum.args$line)){
+        df$line  <<- data.frame[[stratum.args$line]]
+        if(stratum.args$line.label==gettext("variable label")){
+          stratum.args$line.label <- variableLabels[which(stratum.args$line==variableNames)]
+        } else if(stratum.args$line.label==gettext("variable name")){
+          stratum.args$line.label <- stratum.args$line
+        }        
       }
       
       if(facet.args$on){
@@ -135,12 +232,24 @@ setRefClass("RzPlot",
         }
         
       }
+
+      scale_x <- switch(misc.args$scalex,
+                        default=NULL,
+                        get(sprintf("scale_x_%s", misc.args$scalex),
+                            envir=findPackageEnv("package:ggplot2"))
+                        )
+      scale_y <- switch(misc.args$scaley,
+                        default=NULL,
+                        get(sprintf("scale_y_%s", misc.args$scaley),
+                            envir=findPackageEnv("package:ggplot2"))
+                        )
       
-      if(geom.args$na.rm) df <<- na.omit(df)
+      if(misc.args$na.rm) df <<- na.omit(df)
 
       p <- NULL
       
-      if (geom.args$geom == "jitter") {
+#      if (geom.args$geom == "jitter") {
+      if (is.null(position.layer)) {
         p <- qplot(data=df, geom=geom.args$geom)      
       } else {
         p <- qplot(data=df, geom=geom.args$geom, position=position.layer)
@@ -165,7 +274,7 @@ setRefClass("RzPlot",
         
         if (!is.na(binwidth)) {
           p <- p + stat_bin(binwidth=binwidth)
-        } else if (!is.na(breaks)) {
+        } else if (all(!is.na(breaks))) {
           p <- p + stat_bin(breaks=breaks)          
         }        
       }
@@ -201,11 +310,38 @@ setRefClass("RzPlot",
       if(stat.args$stat=="density"){
         p <- p + stat_density()
       } else if(stat.args$stat=="smooth"){
-        p <- p + stat_smooth()
+        p <- p + stat_smooth(method=stat.args$method)
       } else if(stat.args$stat=="quantile"){
         p <- p + stat_quantile()
       } else if(stat.args$stat=="sum"){
-        p <- p + stat_sum(aes(group = 1)) 
+        if(is.null(group)){
+          p <- p + stat_sum(aes(group = 1))
+        } else {
+          p <- p + stat_sum()          
+        }
+      } else if(stat.args$stat=="summary"){
+        if (is.null(position.layer)) {
+          if(is.null(stat.args$color)){
+            p <- p + stat_summary(fun.data=stat.args$fun, geom=stat.args$geom, size=stat.args$size)            
+          } else {
+            p <- p + stat_summary(fun.data=stat.args$fun, geom=stat.args$geom, size=stat.args$size, color=stat.args$color)
+          }
+          
+          if(stat.args$geom2!="none") {
+            p <- p + stat_summary(fun.y=ifelse(stat.args$fun=="median_hilow","median", "mean"),
+                                  geom=stat.args$geom2)
+          }
+        } else {
+          if(is.null(stat.args$color)){
+            p <- p + stat_summary(fun.data=stat.args$fun, geom=stat.args$geom, size=stat.args$size, position=position.layer)            
+          } else {
+            p <- p + stat_summary(fun.data=stat.args$fun, geom=stat.args$geom, size=stat.args$size, color=stat.args$color, position=position.layer)
+          }
+          if(stat.args$geom2!="none") {
+            p <- p + stat_summary(fun.y=ifelse(stat.args$fun=="median_hilow","median", "mean"),
+                                  geom=stat.args$geom2, position=position.layer)
+          }
+        }
       }
       
       
@@ -227,22 +363,34 @@ setRefClass("RzPlot",
         }
       }
       
-      
       # ------------------------ stratum ------------------------
       if(!is.null(stratum.args$group)){
-        p <- p + aes(group=group)  + labs(group=stratum.args$group.label)
+        p <- p + aes(group=group)
       }
       if(!is.null(stratum.args$fill)){
-        p <- p + aes(fill=fill)    + labs(fill=stratum.args$fill.label)
+        scale <- switch(stratum.args$scale,
+                        hue = scale_fill_hue(),
+                        grey= scale_fill_grey(),
+                        scale_fill_brewer(palette=stratum.args$scale))
+        p <- p + scale
+        p <- p + aes(fill=fill)     + labs(fill=stratum.args$fill.label)
       }
       if(!is.null(stratum.args$color)){
-        p <- p + aes(colour=color) + labs(colour=stratum.args$color.label)
+        scale <- switch(stratum.args$scale,
+                        hue = scale_colour_hue(),
+                        grey= scale_colour_grey(),
+                        scale_colour_brewer(palette=stratum.args$scale))
+        p <- p + scale
+        p <- p + aes(colour=color)  + labs(colour=stratum.args$color.label)
       }
       if(!is.null(stratum.args$shape)){
-        p <- p + aes(shape=shape)  + labs(shape=stratum.args$shape.label)
+        p <- p + aes(shape=shape)   + labs(shape=stratum.args$shape.label)
       }
       if(!is.null(stratum.args$size)){
-        p <- p + aes(size=size)    + labs(size=stratum.args$size.label)
+        p <- p + aes(size=size)     + labs(size=stratum.args$size.label)
+      }
+      if(!is.null(stratum.args$line)){
+        p <- p + aes(linetype=line) + labs(linetype=stratum.args$line.label)
       }
       
       
@@ -259,10 +407,15 @@ setRefClass("RzPlot",
       }
       
       
-      # ------------------------ flip ------------------------
-      if(geom.args$flip){
-        p <- p + coord_flip()
+      
+      # ------------------------ scale & coord ------------------------
+      if(!is.null(scale_x)) p <- p + scale_x()
+      if(!is.null(scale_y)) p <- p + scale_y()
+      if(any(c(misc.args$coordx, misc.args$coordy)!="identity")) {
+        p <- p + coord_trans(xtrans=misc.args$coordx, ytrans=misc.args$coordy)        
       }
+      p <- p + coord_cartesian(xlim=misc.args$xlim, ylim=misc.args$ylim)
+      if(misc.args$flip) p <- p + coord_flip()
       
       
       # ------------------------ title ------------------------
@@ -270,7 +423,10 @@ setRefClass("RzPlot",
       
       
       # ------------------------ plot ------------------------
-      p <- p + opts(legend.position=stratum.args$legend.position)
+      
+      p <- p + opts(legend.position=stratum.args$legend.position,
+                    legend.background = theme_rect(fill="white", linetype=stratum.args$legend.linetype),
+                    legend.justification = legend.justification)
       
       con <- textConnection("str", open="w", local=TRUE)
       str <- ""
@@ -281,16 +437,32 @@ setRefClass("RzPlot",
       
       if (nzchar(str)) {
         info.bar$setMessageType(GtkMessageType["warning"])
-        info.bar$setText(paste(str, collapse="\n"))
+        str <- paste(str, collapse="\n")
+        str <- strsplit(str, " ")[[1]]
+        str <- capture.output(cat(str, fill=80))
+        str <- paste(str, collapse="\n")
+        info.bar$setText(str)
         info.bar$show()
         p.current <<- p
+        p.list    <<- c(list(p), p.list)
+        p.current.num <<- 1
+        if(length(p.list) >= 2) button.prev["sensitive"] <<- TRUE
+        button.next["sensitive"] <<- FALSE
         assign("rz.last.plot", p.current, envir=.GlobalEnv)
-      } else if (!is.null(e)) {
+      } else if (!is.list(e)) {
         info.bar$setMessageType(GtkMessageType["error"])
-        info.bar$setText(e[1])
+        str <- paste(e[1], collapse="\n")
+        str <- strsplit(str, " ")[[1]]
+        str <- capture.output(cat(str, fill=80))
+        str <- paste(str, collapse="\n")
+        info.bar$setText(str)
         info.bar$show()
       } else {
         p.current <<- p
+        p.list    <<- c(list(p), p.list)
+        p.current.num <<- 1
+        if(length(p.list) >= 2) button.prev["sensitive"] <<- TRUE
+        button.next["sensitive"] <<- FALSE
         assign("rz.last.plot", p.current, envir=.GlobalEnv)
         info.bar$hide()
       }
@@ -302,6 +474,7 @@ setRefClass("RzPlot",
       geom.args <- geom$getArgs()
       position.args   <-  position$getArgs()
       position.layer <- switch(position.args$position,
+                               default  = NULL,
                                identity = position_identity(width=position.args$width, height= position.args$height),
                                dodge    = position_dodge   (width=position.args$width, height= position.args$height),
                                fill     = position_fill    (width=position.args$width, height= position.args$height),
@@ -310,11 +483,18 @@ setRefClass("RzPlot",
       
       p <- p.current
       
-      if (geom.args$geom == "jitter") {
-        # ------------ geom_jitter ------------
-        p <- p + geom_jitter()
-      } else {
-        geom_selected <- get(sprintf("geom_%s", geom.args$geom))
+#      if (geom.args$geom == "jitter") {
+#        # ------------ geom_jitter ------------
+#        p <- p + geom_jitter()
+#      } else {
+#        geom_selected <- get(sprintf("geom_%s", geom.args$geom))
+#        p <- p + geom_selected(position=position.layer)
+#      }
+      
+      geom_selected <- get(sprintf("geom_%s", geom.args$geom))
+      if(is.null(position.layer)){
+        p <- p + geom_selected()        
+      } else{
         p <- p + geom_selected(position=position.layer)
       }
       
@@ -351,16 +531,32 @@ setRefClass("RzPlot",
       
       if (nzchar(str)) {
         info.bar$setMessageType(GtkMessageType["warning"])
-        info.bar$setText(paste(str, collapse="\n"))
+        str <- paste(str, collapse="\n")
+        str <- strsplit(str, " ")[[1]]
+        str <- capture.output(cat(str, fill=80))
+        str <- paste(str, collapse="\n")
+        info.bar$setText(str)
         info.bar$show()
         p.current <<- p
+        p.list    <<- c(list(p), p.list)
+        p.current.num <<- 1
+        if(length(p.list) >= 2) button.prev["sensitive"] <<- TRUE
+        button.next["sensitive"] <<- FALSE
         assign("rz.last.plot", p.current, envir=.GlobalEnv)
-      } else if (!is.null(e)) {
+      } else if (!is.list(e)) {
         info.bar$setMessageType(GtkMessageType["error"])
-        info.bar$setText(e[1])
+        str <- paste(e[1], collapse="\n")
+        str <- strsplit(str, " ")[[1]]
+        str <- capture.output(cat(str, fill=80))
+        str <- paste(str, collapse="\n")
+        info.bar$setText(str)
         info.bar$show()
       } else {
         p.current <<- p
+        p.list    <<- c(list(p), p.list)
+        p.current.num <<- 1
+        if(length(p.list) >= 2) button.prev["sensitive"] <<- TRUE
+        button.next["sensitive"] <<- FALSE
         assign("rz.last.plot", p.current, envir=.GlobalEnv)
         info.bar$hide()
       }
@@ -375,7 +571,11 @@ setRefClass("RzPlot",
       if(stat.args$stat=="none"){
         return()
       } else if(stat.args$stat=="smooth"){
-        p <- p + stat_smooth()
+        if(is.null(p$mapping$group)){
+          p <- p + stat_smooth(method=stat.args$method, aes(group=1))
+        } else {
+          p <- p + stat_smooth(method=stat.args$method)          
+        }
       } else if(stat.args$stat=="quantile"){
         p <- p + stat_quantile()
       } else if(stat.args$stat=="sum"){
@@ -384,6 +584,19 @@ setRefClass("RzPlot",
         } else {
           p <- p + stat_sum()
         }
+      } else if(stat.args$stat=="summary"){
+        if(is.null(p$mapping$group)) p <- p + aes(group=1)
+        
+        if(is.null(stat.args$color)){
+          p <- p + stat_summary(fun.data=stat.args$fun, geom=stat.args$geom, size=stat.args$size)            
+        } else {
+          p <- p + stat_summary(fun.data=stat.args$fun, geom=stat.args$geom, size=stat.args$size, color=stat.args$color)
+        }
+        
+        if(stat.args$geom2!="none") {
+          p <- p + stat_summary(fun.y=ifelse(stat.args$fun=="median_hilow","median", "mean"),
+                                geom=stat.args$geom2)
+        }
       }
       
       con <- textConnection("str", open="w", local=TRUE)
@@ -395,25 +608,42 @@ setRefClass("RzPlot",
       
       if (nzchar(str)) {
         info.bar$setMessageType(GtkMessageType["warning"])
-        info.bar$setText(paste(str, collapse="\n"))
+        str <- paste(str, collapse="\n")
+        str <- strsplit(str, " ")[[1]]
+        str <- capture.output(cat(str, fill=80))
+        str <- paste(str, collapse="\n")
+        info.bar$setText(str)
         info.bar$show()
         p.current <<- p
+        p.list    <<- c(list(p), p.list)
+        p.current.num <<- 1
+        if(length(p.list) >= 2) button.prev["sensitive"] <<- TRUE
+        button.next["sensitive"] <<- FALSE
         assign("rz.last.plot", p.current, envir=.GlobalEnv)
-      } else if (!is.null(e)) {
+      } else if (!is.list(e)) {
         info.bar$setMessageType(GtkMessageType["error"])
-        info.bar$setText(e[1])
+        str <- paste(e[1], collapse="\n")
+        str <- strsplit(str, " ")[[1]]
+        str <- capture.output(cat(str, fill=80))
+        str <- paste(str, collapse="\n")
+        info.bar$setText(str)
         info.bar$show()
       } else {
         p.current <<- p
+        p.list    <<- c(list(p), p.list)
+        p.current.num <<- 1
+        if(length(p.list) >= 2) button.prev["sensitive"] <<- TRUE
+        button.next["sensitive"] <<- FALSE
         assign("rz.last.plot", p.current, envir=.GlobalEnv)
         info.bar$hide()
       }
     },
     
     setModel = function(model){
+      model <<- model
       geom$completionSetModel(model)
       stratum$completionSetModel(model)
       facet$completionSetModel(model)
     }
 ))
-rzplot$accessors("main", "info.bar")
+rzplot$accessors("main", "info.bar", "data")
